<?php
/**
 * Description of class-pcd-product.
 *
 * @link       orionorigin@orionorigin.com
 * @since      1.0.0
 *
 * @package    Pcd
 * @subpackage Pcd/admin
 */

/**
 *  Description of class-pcd-product.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    Pcd
 * @subpackage Pcd/admin
 * @author     HL
 */
class PCD_Product {

    /**
     * The variation ID of this class.
     *
     * @access   public
     * @var      string    $variation_id    The variation ID of this class.
     */
    public $variation_id;

    /**
     * The ID of this class.
     *
     * @access   public
     * @var      string    $root_product_id The product ID.
     */
    public $root_product_id;

    /**
     * The product.
     *
     * @access   public
     * @var      object    $product The product.
     */
    public $product;

    /**
     * The settings of the product.
     *
     * @access   public
     * @var      array    $settings The settings of the product.
     */
    public $settings;

    /**
     * The settings of the variation.
     *
     * @access   public
     * @var      array    $variation_settings The settings of the variation.
     */
    public $variation_settings;

    /**
     * The constructor of the class
     *
     * @param int $id The id of the product.
     */
    public function __construct($id) {
        if ($id) {
            $this->root_product_id = $this->get_parent($id);
            // If it's a variable product.
            if ($id !== $this->root_product_id) {
                $this->variation_id = $id;
            } else {
                // Simple product and others.
                $this->variation_id = $this->root_product_id;
            }
            $this->product = wc_get_product($id);
            $config = get_post_meta($this->root_product_id, 'pcd-metas', true);

            if (isset($config[$this->variation_id])) {
                $config_id = $config[$this->variation_id]['config-id'];
                if ($config_id) {
                    $this->settings = get_post_meta($config_id, 'pcd-metas', true);
                    $product_metas = get_post_meta($this->root_product_id, 'pcd-metas', true);
                    if (isset($product_metas['related-products'])) {
                        $this->settings['related-products'] = $product_metas['related-products'];
                    }
                    if (isset($product_metas['related-quantities'])) {
                        $this->settings['related-quantities'] = $product_metas['related-quantities'];
                    }
                }
            }
        }
    }

    /**
     * Get usable attribut and extract.
     */
    public function extract_usable_attributes() {
        $product = $this->product;
        $attributes = $product->get_attributes();
        $usable_attributes = array();
        foreach ($attributes as $attribute) {
            $sanitized_name = sanitize_title($attribute['name']);
            if ($attribute['is_visible'] && $attribute['is_variation']) {
                if ($attribute['is_taxonomy']) {
                    $values = wc_get_product_terms($product->get_id(), $attribute['name'], array('fields' => 'all'));
                    $taxonomy = get_taxonomy($attribute['name']);
                    $key = 'attribute_' . $sanitized_name;
                    $usable_attributes[$attribute['name']] = array(
                        'key' => $key,
                        'label' => $taxonomy->labels->singular_name,
                        'values' => $values,
                    );
                } else {
                    $key = 'attribute_' . $sanitized_name;
                    $values = array_map('trim', explode(WC_DELIMITER, $attribute['value']));
                    $usable_attributes[$attribute['name']] = array(
                        'key' => $key,
                        'label' => $attribute['name'],
                        'values' => $values,
                    );
                }
            }
        }

        return $usable_attributes;
    }

    /**
     * Checks the product contains at least one active part
     *
     * @return boolean
     */
    public function has_model() {
        $models = get_proper_value($this->settings, 'models');
        return !empty($models);
    }

    /**
     * Save product settings fields.
     *
     * @param int $item_id The item id.
     */
    public function save_product_settings_fields($item_id) {
        $meta_key = 'pcd-metas';
        $variation = wc_get_product($item_id);
        // If we're dealing with a variation, Product ID is the root ID of the product.
        if ('WC_Product_Variation' === get_class($variation)) {
            $product_id = $variation->get_parent_id();
        } else {
            $product_id = $item_id;
        }
        if (isset($_POST[$meta_key])) {
            // Careful this hooks only send the updated data, not the complete form.
            $old_metas = get_post_meta($product_id, $meta_key, true);
            if (empty($old_metas)) {
                $old_metas = array();
            }
            $new_metas = array_replace($old_metas, $_POST[$meta_key]);

            // If the related products and quantities are not in the post variable, that means the user is disabling them.
            if (!isset($_POST[$meta_key]['related-products'])) {
                $new_metas['related-products'] = array();
            }

            if (!isset($_POST[$meta_key]['related-quantities'])) {
                $new_metas['related-quantities'] = array();
            }
        } else {
            $new_metas = array();
            $new_metas['related-products'] = array();
            $new_metas['related-quantities'] = array();
        }
        update_post_meta($product_id, $meta_key, $new_metas);
    }

    /**
     * Get notice if is variable product.
     */
    public function get_variable_product_details_location_notice() {
        ?>
        <div class="options_group show_if_simple show_if_variable">
            <p class="form-field _sold_individually_field show_if_simple show_if_variable" style="background: #00a0d2;color: white;display: block;padding:  0 !important;padding-left:  10px !important;">
                <?php __('In order to assign a configuration to a variation, you will need to go into the variation properties (same area you define a variation price).', 'pcd'); ?>
            </p>
        </div>
        <?php
    }

    /**
     * Save configuration on product page.
     *
     * @param int $post_id The id of post to save.
     */
    public function save_config($post_id) {
        $meta_key = 'pcd-metas';
        if (isset($_POST[$meta_key])) {
            update_post_meta($post_id, $meta_key, $_POST[$meta_key]);
        }
    }

    /**
     * Returns the defined value for a product setting which can be local(product metas) or global (options)
     *
     * @param array  $product_settings Product options.
     * @param array  $global_settings Global options.
     * @param string $option_name Option name / Meta key.
     * @param int    $field_value Default value to return if empty.
     * @return string
     */
    public function get_option($product_settings, $global_settings, $option_name, $field_value = '') {
        if (isset($product_settings[$option_name]) && ( (!empty($product_settings[$option_name]) ) || '0' === $product_settings[$option_name] )) {
            $field_value = $product_settings[$option_name];
        } elseif (isset($global_settings[$option_name]) && !empty($global_settings[$option_name])) {
            $field_value = $global_settings[$option_name];
        }

        return $field_value;
    }

    /**
     * Returns a variation root product ID
     *
     * @param type $variation_id Variation ID.
     * @return int
     */
    public function get_parent($variation_id) {
        $variable_product = wc_get_product($variation_id);
        if (!$variable_product) {
            return false;
        }
        if ('variation' !== $variable_product->get_type()) {
            $product_id = $variation_id;
        } else {
            $product_id = $variable_product->get_parent_id();
        }
        return $product_id;
    }

    function get_buttons($with_upload = false) {
        ob_start();
        $content = '';
        $product = $this->product;

        if ($this->variation_id) {
            $item_id = $this->variation_id;
        } else {
            $item_id = $this->root_product_id;
        }
        if ('variation' !== $product->get_type()) {
            pcd_generate_design_buttons_css();
        }
        if ('variable' === $product->get_type()) {
            $variations = $product->get_available_variations();
            foreach ($variations as $variation) {
                if (!$variation['is_purchasable'] || !$variation['is_in_stock']) {
                    continue;
                }
                $pcd_product = new PCD_Product($variation['variation_id']);

                echo $pcd_product->get_buttons($with_upload);
            }
        } else {
            
            $config = get_post_meta($this->root_product_id, 'pcd-metas', true);
            
            if (!isset($config[$this->variation_id]['config-id']) || empty($config[$this->variation_id]['config-id']) || !$this->has_model()) {
                $output = ob_get_clean();
                return $output;
            }
            
            ?>

            <div class="pcd-buttons-wrap-<?php echo $product->get_type(); ?>" data-id="<?php echo $this->variation_id; ?>">
                <?php
                // Design from blank
                

                $design_from_blank_url = $this->get_design_url();
                $content .= '<a  href="' . $design_from_blank_url . '" class="mg-top-10 pcd-customize-product">' . __('Design now', 'pcd') . '</a>';



                if (!isset($item_id)) {
                    $item_id = '';
                }
                if (!isset($design_from_blank_url)) {
                    $design_from_blank_url = '';
                }
                echo $content;
                ?>
            </div>
            <?php
        }

        $output = ob_get_clean();

        return $output;
    }

    public function get_design_url($design_index = false, $cart_item_key = false, $order_item_id = false, $tpl_id = false) {
        global $pcd_settings;

        if ($this->variation_id) {
            $item_id = $this->variation_id;
        } else {
            $item_id = $this->root_product_id;
        }
        $options = $pcd_settings['pcd-general-options'];
        $pcd_page_id = $options['pcd_page_id'];
        if (function_exists('icl_object_id')) {
            $pcd_page_id = icl_object_id($pcd_page_id, 'page', false, ICL_LANGUAGE_CODE);
        }
        $pcd_page_url = '';
        if ($pcd_page_id) {
            $pcd_page_url = get_permalink($pcd_page_id);
            if ($item_id) {
                $query = parse_url($pcd_page_url, PHP_URL_QUERY);
                // Returns a string if the URL has parameters or NULL if not
                if (get_option('permalink_structure')) {
                    if (substr($pcd_page_url, -1) != '/') {
                        $pcd_page_url .= '/';
                    }
                    if ($design_index || $design_index === 0) {
                        $pcd_page_url .= "saved-design/$item_id/$design_index/";
                    } elseif ($cart_item_key) {
                        $pcd_page_url .= "edit/$item_id/$cart_item_key/";
                    } elseif ($order_item_id) {
                        $pcd_page_url .= "ordered-design/$item_id/$order_item_id/";
                    } else {
                        $pcd_page_url .= 'design/' . $item_id . '/';
                        if ($tpl_id) {
                            $pcd_page_url .= "$tpl_id/";
                        }
                    }
                } else {
                    if ($design_index !== false) {
                        $pcd_page_url .= '&product_id=' . $item_id . '&design_index=' . $design_index;
                    } elseif ($cart_item_key) {
                        $pcd_page_url .= '&product_id=' . $item_id . '&edit=' . $cart_item_key;
                    } elseif ($order_item_id) {
                        $pcd_page_url .= '&product_id=' . $item_id . '&oid=' . $order_item_id;
                    } else {
                        $pcd_page_url .= '&product_id=' . $item_id;
                        if ($tpl_id) {
                            $pcd_page_url .= "&tpl=$tpl_id";
                        }
                    }
                }
            }
        }

        return $pcd_page_url;
    }

    public function is_customizable() {
        return (!empty($this->settings) );
    }
  /**
     * Returns the minimum and maximum order quantities
     *
     * @return type
     */
    public function get_purchase_properties() {
        if ($this->variation_id) {
            $defined_min_qty = get_post_meta($this->variation_id, 'variation_minimum_allowed_quantity', true);
            // We consider the values defined for the all of them
            if (!$defined_min_qty) {
                $defined_min_qty = get_post_meta($this->root_product_id, 'minimum_allowed_quantity', true);
            }
            $product_metas = get_post_meta($this->root_product_id, 'pcd-metas', true);

            if (!$defined_min_qty && isset($product_metas['related-products']) && !empty($product_metas['related-products'])) {
                $defined_min_qty = 0;
            } elseif (!isset($product_metas['related-products']) || empty($product_metas['related-products'])) {
                $defined_min_qty = 1;
            }

            $defined_max_qty = get_post_meta($this->variation_id, 'variation_maximum_allowed_quantity', true);
            // We consider the values defined for the all of them
            if (!$defined_max_qty) {
                $defined_max_qty = get_post_meta($this->root_product_id, 'maximum_allowed_quantity', true);
            }
        } else {
            $defined_min_qty = get_post_meta($this->root_product_id, 'minimum_allowed_quantity', true);
            if (!$defined_min_qty) {
                $defined_min_qty = 1;
            }

            $defined_max_qty = get_post_meta($this->root_product_id, 'variation_maximum_allowed_quantity', true);
        }

        if (!$defined_max_qty) {
            $defined_max_qty = apply_filters('woocommerce_quantity_input_max', $this->product->backorders_allowed() ? '' : $this->product->get_stock_quantity(), $this->product);
        }

        $min_to_purchase = $defined_min_qty;
        if (!$defined_min_qty) {
            $min_to_purchase = 1;
        }

        $defaults = array(
            'max_value' => $defined_max_qty,
            'min_value' => $defined_min_qty,
            'step' => '1',
        );
        $args = apply_filters('woocommerce_quantity_input_args', wp_parse_args(array(), $defaults), $this->product);

        return array(
            'min' => $args['min_value'],
            'min_to_purchase' => $args['min_value'],
            'max' => $args['max_value'],
            'step' => $args['step'],
        );
    }
}
