<?php

/**
 * The file that defines the core plugin class
 *
 * A class definition that includes attributes and functions used across both the
 * public-facing side of the site and the admin area.
 *
 * @link       orionorigin@orionorigin.com
 * @since      1.0.0
 *
 * @package    Pcd
 * @subpackage Pcd/includes
 */

/**
 * The core plugin class.
 *
 * This is used to define internationalization, admin-specific hooks, and
 * public-facing site hooks.
 *
 * Also maintains the unique identifier of this plugin as well as the current
 * version of the plugin.
 *
 * @since      1.0.0
 * @package    Pcd
 * @subpackage Pcd/includes
 * @author     orionorigin <orionorigin@orionorigin.com>
 */
class Pcd {

    /**
     * The loader that's responsible for maintaining and registering all hooks that power
     * the plugin.
     *
     * @since    1.0.0
     * @access   protected
     * @var      Pcd_Loader    $loader    Maintains and registers all hooks for the plugin.
     */
    protected $loader;

    /**
     * The unique identifier of this plugin.
     *
     * @since    1.0.0
     * @access   protected
     * @var      string    $plugin_name    The string used to uniquely identify this plugin.
     */
    protected $plugin_name;

    /**
     * The current version of the plugin.
     *
     * @since    1.0.0
     * @access   protected
     * @var      string    $version    The current version of the plugin.
     */
    protected $version;

    /**
     * Define the core functionality of the plugin.
     *
     * Set the plugin name and the plugin version that can be used throughout the plugin.
     * Load the dependencies, define the locale, and set the hooks for the admin area and
     * the public-facing side of the site.
     *
     * @since    1.0.0
     */
    public function __construct() {
        if (defined('PLUGIN_NAME_VERSION')) {
            $this->version = PLUGIN_NAME_VERSION;
        } else {
            $this->version = '1.0.0';
        }
        $this->plugin_name = 'pcd';

        $this->load_dependencies();
        $this->set_locale();
        $this->define_admin_hooks();
        $this->define_public_hooks();
    }

    /**
     * Load the required dependencies for this plugin.
     *
     * Include the following files that make up the plugin:
     *
     * - Pcd_Loader. Orchestrates the hooks of the plugin.
     * - Pcd_I18n. Defines internationalization functionality.
     * - Pcd_Admin. Defines all hooks for the admin area.
     * - Pcd_Public. Defines all hooks for the public side of the site.
     *
     * Create an instance of the loader which will be used to register the hooks
     * with WordPress.
     *
     * @since    1.0.0
     * @access   private
     */
    private function load_dependencies() {

        /**
         * The class responsible for orchestrating the actions and filters of the
         * core plugin.
         */
        require_once plugin_dir_path(dirname(__FILE__)) . 'includes/class-pcd-loader.php';

        /**
         * The class responsible for defining internationalization functionality
         * of the plugin.
         */
        require_once plugin_dir_path(dirname(__FILE__)) . 'includes/class-pcd-i18n.php';

        /**
         * The class responsible for defining all actions that occur in the admin area.
         */
        require_once plugin_dir_path(dirname(__FILE__)) . 'admin/class-pcd-admin.php';

        /**
         * The class responsible for defining all actions that occur in the public-facing
         * side of the site.
         */
        require_once plugin_dir_path(dirname(__FILE__)) . 'public/class-pcd-public.php';

        /**
         * The class responsible for defining all actions that occur in the admin area and related to products.
         */
        require_once plugin_dir_path(dirname(__FILE__)) . 'admin/class-pcd-product.php';

        $this->loader = new Pcd_Loader();
    }

    /**
     * Define the locale for this plugin for internationalization.
     *
     * Uses the Pcd_I18n class in order to set the domain and to register the hook
     * with WordPress.
     *
     * @since    1.0.0
     * @access   private
     */
    private function set_locale() {

        $plugin_i18n = new Pcd_I18n();

        $this->loader->add_action('plugins_loaded', $plugin_i18n, 'load_plugin_textdomain');
    }

    /**
     * Register all of the hooks related to the admin area functionality
     * of the plugin.
     *
     * @since    1.0.0
     * @access   private
     */
    private function define_admin_hooks() {

        $plugin_admin = new Pcd_Admin($this->get_plugin_name(), $this->get_version());

        $this->loader->add_action('admin_enqueue_scripts', $plugin_admin, 'enqueue_styles');
        $this->loader->add_action('admin_enqueue_scripts', $plugin_admin, 'enqueue_scripts');
        $this->loader->add_action('admin_menu', $plugin_admin, 'add_pcd_submenu');
        $this->loader->add_action('admin_init', $plugin_admin, 'pcd_redirect');


        // General.
        $this->loader->add_action('init', $plugin_admin, 'init_globals');
        $this->loader->add_action('admin_notices', $plugin_admin, 'notify_customization_page_missing');
        $this->loader->add_action('admin_notices', $plugin_admin, 'notify_minmimum_required_parameters');
        $this->loader->add_action('admin_notices', $plugin_admin, 'get_help_notices');
        //$this->loader->add_action('admin_notices', $plugin_admin, 'get_missing_models_notice');
        $this->loader->add_filter('upload_mimes', $plugin_admin, 'pcd_add_custom_mime_types');
        $this->loader->add_action('admin_notices', $plugin_admin, 'get_max_input_vars_php_ini');

        // Pcd post types hooks hooks.
        $pcd_post = new PCD_Post_Types();
        $this->loader->add_action('init', $pcd_post, 'register_cpt_cliparts');
        $this->loader->add_action('init', $pcd_post, 'register_cpt_config');
        $this->loader->add_action('add_meta_boxes', $pcd_post, 'get_cliparts_metabox');
        $this->loader->add_action('add_meta_boxes', $pcd_post, 'get_config_metabox');
        $this->loader->add_filter('get_user_option_meta-box-order_pcd-config', $pcd_post, 'get_metabox_order');
        $this->loader->add_filter('manage_edit-product_columns', $pcd_post, 'get_product_columns');
        $this->loader->add_action('manage_product_posts_custom_column', $pcd_post, 'get_products_columns_values', 5, 2);


        // Cliparts hooks.
        $clipart = new Pcd_Clipart();
        $this->loader->add_action('save_post_pcd-cliparts', $clipart, 'save_cliparts');

        // Configurations hooks.
        $pcd_config = new PCD_Config();
        $this->loader->add_action('save_post_pcd-config', $pcd_config, 'save_config');
        $this->loader->add_action('save_post_product', $pcd_config, 'save_config');
        $this->loader->add_action('woocommerce_product_options_general_product_data', $pcd_config, 'get_product_config_selector');
        $this->loader->add_action('woocommerce_product_after_variable_attributes', $pcd_config, 'get_variation_product_config_selector', 10, 3);
        $this->loader->add_action('admin_action_pcd_duplicate_config', $pcd_config, 'pcd_duplicate_config');
        $this->loader->add_filter('post_row_actions', $pcd_config, 'get_duplicate_post_link', 10, 2);
        $this->loader->add_action('woocommerce_save_product_variation', $pcd_config, 'save_variation_settings_fields');

        // Products.
        $product_admin = new PCD_Product(false);
        $this->loader->add_action('save_post_product', $product_admin, 'save_product_settings_fields');
        $this->loader->add_action('woocommerce_save_product_variation', $product_admin, 'save_product_settings_fields');
        $this->loader->add_action('woocommerce_product_options_inventory_product_data', $product_admin, 'get_variable_product_details_location_notice');

        // subscription.
        $this->loader->add_action('admin_notices', $plugin_admin, 'pcd_get_subscription_notice');

        // subscription hook.
        $this->loader->add_action('wp_ajax_pcd_subscribe', $plugin_admin, 'pcd_subscribe');
        $this->loader->add_action('wp_ajax_nopriv_pcd_subscribe', $plugin_admin, 'pcd_subscribe');

        // hide subscription hook.
        $this->loader->add_action('wp_ajax_pcd_hide_notice', $plugin_admin, 'pcd_hide_notice');
        $this->loader->add_action('wp_ajax_nopriv_pcd_hide_notice', $plugin_admin, 'pcd_hide_notice');
    }

    /**
     * Register all of the hooks related to the public-facing functionality
     * of the plugin.
     *
     * @since    1.0.0
     * @access   private
     */
    private function define_public_hooks() {

        $plugin_public = new Pcd_Public($this->get_plugin_name(), $this->get_version());

        $this->loader->add_action('wp_enqueue_scripts', $plugin_public, 'enqueue_styles');
        $this->loader->add_action('wp_enqueue_scripts', $plugin_public, 'enqueue_scripts');
        $this->loader->add_action('woocommerce_after_add_to_cart_button', $plugin_public, 'get_customize_btn');
        $this->loader->add_filter('woocommerce_loop_add_to_cart_link', $plugin_public, 'get_customize_btn_loop', 10, 2);
        // Add query vars and rewrite rules
        $this->loader->add_filter('query_vars', $plugin_public, 'pcd_add_query_vars');
        $this->loader->add_filter('init', $plugin_public, 'pcd_add_rewrite_rules', 99);
        $this->loader->add_action('init', $plugin_public, 'set_variable_action_filters', 99);
        $this->loader->add_action('woocommerce_after_add_to_cart_button', $plugin_public, 'hide_cart_button');
        $this->loader->add_filter('post_class', $plugin_public, 'get_item_class', 10, 3);
        $this->loader->add_action('init', $plugin_public, 'register_shortcodes');

        // Variable filters
        $this->loader->add_action('init', $plugin_public, 'set_variable_action_filters', 99);

        // Handle user pictures upload.
        $this->loader->add_action('wp_ajax_pcd_handle_picture_upload', $plugin_public, 'pcd_handle_picture_upload');
        $this->loader->add_action('wp_ajax_nopriv_pcd_handle_picture_upload', $plugin_public, 'pcd_handle_picture_upload');

        // Handle delete user pictures upload.
        $this->loader->add_action('wp_ajax_pcd_handle_delete_picture_upload', $plugin_public, 'pcd_handle_delete_picture_upload');
        $this->loader->add_action('wp_ajax_nopriv_pcd_handle_delete_picture_upload', $plugin_public, 'pcd_handle_delete_picture_upload');

        // Move user pictures upload from cookie in user account.
        $this->loader->add_action('user_register', $plugin_public, 'move_user_picture_uplaods', 10, 1);

        // Social login.
        $this->loader->add_action('init', $plugin_public, 'process_social_login', 99);
        $pcd_design = new PCD_Design();
        //Save design for later.
        $this->loader->add_action('wp_ajax_save_design_for_later', $pcd_design, 'save_design_for_later_ajax');
        $this->loader->add_action('wp_ajax_nopriv_save_design_for_later', $pcd_design, 'save_design_for_later_ajax');

        //Get user saved designs in acount.
        $this->loader->add_action('woocommerce_before_my_account', $pcd_design, 'get_user_saved_designs');

        //Delete saved designs.
        $this->loader->add_action('wp_ajax_delete_saved_design', $pcd_design, 'delete_saved_design_ajax');
        $this->loader->add_action('wp_ajax_nopriv_delete_saved_design', $pcd_design, 'delete_saved_design_ajax');

        //Generate downloadable file. Download design.
        $this->loader->add_action('wp_ajax_generate_downloadable_file', $pcd_design, 'generate_downloadable_file');
        $this->loader->add_action('wp_ajax_nopriv_generate_downloadable_file', $pcd_design, 'generate_downloadable_file');

        //Get design price.
        $this->loader->add_action('wp_ajax_get_design_price', $pcd_design, 'get_design_price_ajax');
        $this->loader->add_action('wp_ajax_nopriv_get_design_price', $pcd_design, 'get_design_price_ajax');

        //Add  custom design to cart.
        $this->loader->add_action('wp_ajax_add_custom_design_to_cart', $pcd_design, 'add_custom_design_to_cart_ajax');
        $this->loader->add_action('wp_ajax_nopriv_add_custom_design_to_cart', $pcd_design, 'add_custom_design_to_cart_ajax');

        //Save variation attributes in transients
        $this->loader->add_action('wp_ajax_pcd_store_variation_attributes', $plugin_public, 'pcd_store_variation_attributes');
        $this->loader->add_action('wp_ajax_nopriv_pcd_store_variation_attributes', $plugin_public, 'pcd_store_variation_attributes');

        //Save user designs temporary.
        $this->loader->add_action('wp_login', $pcd_design, 'save_user_temporary_designs', 10, 2);

        //Save customixed item meta.
        $this->loader->add_action('woocommerce_new_order_item', $pcd_design, 'save_customized_item_meta', 10, 3);

        $this->loader->add_action('woocommerce_after_order_itemmeta', $pcd_design, 'get_order_custom_admin_data', 10, 3);
        $this->loader->add_filter('woocommerce_email_attachments', $pcd_design, 'add_order_design_to_mail', 10, 3);

        // User my account page
        $this->loader->add_action('woocommerce_order_item_meta_end', $pcd_design, 'get_user_account_products_meta', 11, 4);
        $this->loader->add_action('woocommerce_before_calculate_totals', $pcd_design, 'get_cart_item_price', 10);

        // Reload an order
        $this->loader->add_action('woocommerce_my_account_my_orders_actions', $pcd_design, 'get_user_account_load_order_button', 10, 2);


        // Allow us to hide the wpc_data_upl meta from the meta list in the order details page
        $this->loader->add_filter('woocommerce_hidden_order_itemmeta', $pcd_design, 'unset_pcd_data_upl_meta');
        $this->loader->add_filter('woocommerce_add_cart_item_data', $pcd_design, 'force_individual_cart_items', 10, 2);
    }

    /**
     * Run the loader to execute all of the hooks with WordPress.
     *
     * @since    1.0.0
     */
    public function run() {
        $this->loader->run();
    }

    /**
     * The name of the plugin used to uniquely identify it within the context of
     * WordPress and to define internationalization functionality.
     *
     * @since     1.0.0
     * @return    string    The name of the plugin.
     */
    public function get_plugin_name() {
        return $this->plugin_name;
    }

    /**
     * The reference to the class that orchestrates the hooks with the plugin.
     *
     * @since     1.0.0
     * @return    Pcd_Loader    Orchestrates the hooks of the plugin.
     */
    public function get_loader() {
        return $this->loader;
    }

    /**
     * Retrieve the version number of the plugin.
     *
     * @since     1.0.0
     * @return    string    The version number of the plugin.
     */
    public function get_version() {
        return $this->version;
    }

}
